import argparse
import os
import subprocess
import sys

import cascajal.profile


def _get_arg_parser():
    """Create the CLI arg parser for Cascajal."""

    parser = argparse.ArgumentParser(
        description='Generate PDF output for Hieroglyph presentations.',
    )
    parser.add_argument('--firefox',
                        dest='firefox',
                        action='store',
                        help='Location of firefox executable (default: guess)',
    )
    parser.add_argument('-o', '--output',
                        dest='output_filename',
                        action='store',
                        default='slides.pdf',
                        type=lambda s:os.path.abspath(os.path.expanduser(s)),
                        help='Output filename to write (default: slides.pdf)',
    )

    parser.add_argument('input_file',
                        metavar='presentation.html',
                        type=lambda s:os.path.abspath(os.path.expanduser(s)),
                        nargs=1,
                        help='Presentation HTML file generated by Hieroglyph',
    )

    return parser


def main():

    parser = _get_arg_parser()
    args = parser.parse_args()

    if args.firefox is None:
        args.firefox = cascajal.profile.find_firefox()

    if args.firefox is None:
        print("Unable to locate Firefox; try running with --firefox.")
        sys.exit(1)

    with cascajal.profile.Profile(vars(args)) as profile:

        cmdline = [
            args.firefox,
            '-no-remote',
            '-profile',
            profile.location,
            'file://%s' % (args.input_file[0],),
        ]

        print("Preparing to run Firefox; when Firefox starts, "
              "press CTRL-P to generate your PDF."
        )

        print args
        subprocess.call(cmdline)



import os
import pkgutil
import shutil
import tempfile


def find_firefox():
    """Attempt to find a Firefox-like executable and return the path."""

    candidates = [
        '/usr/bin/firefox',
        '/usr/bin/iceweasel',
    ]

    for c in candidates:
        if os.path.exists(c):
            return c


class Profile(object):

    TEMPLATE_FILES = [
        'prefs.js',
        'extensions.ini',
    ]

    def __init__(self, opts):

        self.options = opts

    def __enter__(self):

        # create the temporary profile directory
        self.location = tempfile.mkdtemp()

        os.mkdir(
            os.path.join(self.location, 'extensions')
        )

        file(
            os.path.join(
                self.location,
                'extensions',
                'jid1-bwQXQ9ZfPZiqdw@jetpack.xpi',
            ),
            'wb',
        ).write(
            pkgutil.get_data('cascajal', 'data/prontoprint/prontoprint.xpi'),
        )

        context = self.options.copy()
        context['profile_dir'] = self.location

        # copy templated files into it
        for filename in self.TEMPLATE_FILES:
            file(os.path.join(self.location, filename), 'w').write(
                pkgutil.get_data('cascajal', 'data/%s' % (filename,)) % context
            )

        return self

    def __exit__(self, *exc_info):

        shutil.rmtree(self.location)

        return None



from setuptools import setup, find_packages
import sys, os

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.rst')).read()
NEWS = open(os.path.join(here, 'NEWS.txt')).read()


version = '0.1'

install_requires = [
]


setup(name='cascajal',
      version=version,
      description="PDF generation for Hieroglyph presentations.",
      long_description=README + '\n\n' + NEWS,
      classifiers=[
        'License :: OSI Approved :: BSD License',
        'Topic :: Documentation',
        'Topic :: Text Processing',
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 3",
      ],
      keywords='PDF presentation text-processing',
      author='Nathan R. Yergler',
      author_email='nathan@yergler.net',
      url='https://github.com/nyergler/cascajal',
      license='BSD',
      packages=find_packages('src'),
      package_dir={'': 'src'},
      include_package_data=True,
      package_data={
          'cascajal': [
              'data/*',
          ],
      },
      zip_safe=True,
      install_requires=install_requires,
      entry_points={
          'console_scripts': [
              'cascajal=cascajal.cli:main',
          ],
      },
      test_suite='cascajal.tests',
)



import os
from unittest import TestCase

import cascajal.profile


class ProfileManagerTests(TestCase):

    STUB_OPTS = {
        'output_filename': '',
    }

    def test_creates_dir_with_prefs(self):

        with cascajal.profile.Profile(self.STUB_OPTS) as profile:

            self.assertTrue(
                os.path.isdir(profile.location),
            )
            self.assertTrue(
                os.path.exists(
                    os.path.join(profile.location, 'prefs.js'),
                ),
            )

    def test_cleans_up_on_exit(self):

        location = None

        with cascajal.profile.Profile(self.STUB_OPTS) as profile:

            location = profile.location
            self.assertTrue(
                os.path.isdir(location),
            )

        self.assertFalse(
            os.path.exists(location)
        )

    def test_expands_prefs(self):

        with cascajal.profile.Profile({
            'output_filename': 'test_expands_prefs.pdf',
        }) as profile:

            self.assertTrue(
                os.path.isdir(profile.location),
            )
            self.assertIn(
                'test_expands_prefs.pdf',
                file(
                    os.path.join(profile.location, 'prefs.js'),
                ).read(),
            )
